@@ -1,3 +1,5 @@
+#[cfg(feature = "luau")]
+use full_moon::ast::types::IfExpression;
 use full_moon::{
     ast::{
         span::ContainedSpan, BinOp, Call, Expression, Index, Prefix, Suffix, UnOp, Value, Var,
@@ -283,6 +285,17 @@ pub fn format_suffix(
     }
 }
 
+/// Formats an [`IfExpression`] node
+#[cfg(feature = "luau")]
+fn format_if_expression(
+    _ctx: &Context,
+    if_expression: &IfExpression,
+    _shape: Shape,
+) -> IfExpression {
+    // TODO: Apply actual formatting here
+    if_expression.to_owned()
+}
+
 /// Formats a Value Node
 pub fn format_value(ctx: &Context, value: &Value, shape: Shape) -> Value {
     match value {
@@ -292,6 +305,10 @@ pub fn format_value(ctx: &Context, value: &Value, shape: Shape) -> Value {
         Value::FunctionCall(function_call) => {
             Value::FunctionCall(format_function_call(ctx, function_call, shape))
         }
+        #[cfg(feature = "luau")]
+        Value::IfExpression(if_expression) => {
+            Value::IfExpression(format_if_expression(ctx, if_expression, shape))
+        }
         Value::Number(token_reference) => {
             Value::Number(format_token_reference(ctx, token_reference, shape))
         }
@@ -420,17 +437,17 @@ fn binop_expression_length(expression: &Expression, top_binop: &BinOp) -> usize
                 if binop.is_right_associative() {
                     binop_expression_length(rhs, top_binop)
                         + strip_trivia(binop).to_string().len() + 2 // 2 = space before and after binop
-                        + lhs.to_string().len()
+                        + strip_trivia(&**lhs).to_string().len()
                 } else {
                     binop_expression_length(lhs, top_binop)
                         + strip_trivia(binop).to_string().len() + 2 // 2 = space before and after binop
-                        + rhs.to_string().len()
+                        + strip_trivia(&**rhs).to_string().len()
                 }
             } else {
                 0
             }
         }
-        _ => expression.to_string().len(),
+        _ => strip_trivia(expression).to_string().len(),
     }
 }
 
