@@ -598,6 +598,13 @@ pub fn format_function_body(
     // `function() end`
     let block_empty = trivia_util::is_function_empty(function_body);
 
+    #[cfg(feature = "luau")]
+    let generics = function_body
+        .generics()
+        .map(|generic_declaration| format_generic_declaration(ctx, generic_declaration, shape));
+    #[cfg(feature = "luau")]
+    let shape = shape + generics.as_ref().map_or(0, |x| x.to_string().len());
+
     // Check if the parameters should be placed across multiple lines
     let multiline_params = {
         #[cfg(feature = "luau")]
@@ -734,6 +741,7 @@ pub fn format_function_body(
 
     #[cfg(feature = "luau")]
     let function_body = function_body
+        .with_generics(generics)
         .with_type_specifiers(type_specifiers)
         .with_return_type(return_type);
 
@@ -894,24 +902,12 @@ pub fn format_function_declaration(
     .update_leading_trivia(FormatTriviaType::Append(leading_trivia));
     let formatted_function_name = format_function_name(ctx, function_declaration.name(), shape);
 
-    #[cfg(feature = "luau")]
-    let generics = function_declaration
-        .generics()
-        .map(|generic_declaration| format_generic_declaration(ctx, generic_declaration, shape));
-
     let shape = shape + (9 + strip_trivia(&formatted_function_name).to_string().len()); // 9 = "function "
-    #[cfg(feature = "luau")]
-    let shape = shape + generics.as_ref().map_or(0, |x| x.to_string().len());
     let function_body = format_function_body(ctx, function_declaration.body(), true, shape);
 
-    let function_declaration = FunctionDeclaration::new(formatted_function_name)
+    FunctionDeclaration::new(formatted_function_name)
         .with_function_token(function_token)
-        .with_body(function_body);
-
-    #[cfg(feature = "luau")]
-    let function_declaration = function_declaration.with_generics(generics);
-
-    function_declaration
+        .with_body(function_body)
 }
 
 /// Formats a LocalFunction node
@@ -928,25 +924,13 @@ pub fn format_local_function(
     let function_token = fmt_symbol!(ctx, local_function.function_token(), "function ", shape);
     let formatted_name = format_token_reference(ctx, local_function.name(), shape);
 
-    #[cfg(feature = "luau")]
-    let generics = local_function
-        .generics()
-        .map(|generic_declaration| format_generic_declaration(ctx, generic_declaration, shape));
-
     let shape = shape + (6 + 9 + strip_trivia(&formatted_name).to_string().len()); // 6 = "local ", 9 = "function "
-    #[cfg(feature = "luau")]
-    let shape = shape + generics.as_ref().map_or(0, |x| x.to_string().len());
     let function_body = format_function_body(ctx, local_function.body(), true, shape);
 
-    let local_function = LocalFunction::new(formatted_name)
+    LocalFunction::new(formatted_name)
         .with_local_token(local_token)
         .with_function_token(function_token)
-        .with_body(function_body);
-
-    #[cfg(feature = "luau")]
-    let local_function = local_function.with_generics(generics);
-
-    local_function
+        .with_body(function_body)
 }
 
 /// Formats a MethodCall node
