@@ -1,5 +1,8 @@
 #[cfg(feature = "luau")]
-use full_moon::ast::types::{IndexedTypeInfo, TypeAssertion, TypeField, TypeInfo, TypeSpecifier};
+use full_moon::ast::types::{
+    GenericDeclaration, GenericDeclarationParameter, IfExpression, IndexedTypeInfo, TypeArgument,
+    TypeAssertion, TypeField, TypeFieldKey, TypeInfo, TypeSpecifier,
+};
 use full_moon::ast::{
     punctuated::Punctuated, span::ContainedSpan, BinOp, Call, Expression, FunctionArgs,
     FunctionBody, FunctionCall, FunctionName, Index, MethodCall, Parameter, Prefix, Suffix,
@@ -9,60 +12,60 @@ use full_moon::tokenizer::{Token, TokenReference};
 
 /// Enum to determine how trivia should be added when using trivia formatter functions
 #[derive(Clone, Debug)]
-pub enum FormatTriviaType<'ast> {
+pub enum FormatTriviaType {
     /// Trivia will be added to the end of the current trivia
-    Append(Vec<Token<'ast>>),
+    Append(Vec<Token>),
     /// The current trivia will be replaced with the new trivia
-    Replace(Vec<Token<'ast>>),
+    Replace(Vec<Token>),
     /// Trivia will not be changed
     NoChange,
 }
 
 /// Strips all leading and trailing trivia from a specific node.
 /// This is useful if we need to use the node to calculate sizing, whilst we do not want trivia included
-pub fn strip_trivia<'ast, T>(item: &T) -> T
+pub fn strip_trivia<T>(item: &T) -> T
 where
-    T: UpdateLeadingTrivia<'ast> + UpdateTrailingTrivia<'ast>,
+    T: UpdateLeadingTrivia + UpdateTrailingTrivia,
 {
     item.update_leading_trivia(FormatTriviaType::Replace(vec![]))
         .update_trailing_trivia(FormatTriviaType::Replace(vec![]))
 }
 
-pub fn strip_leading_trivia<'ast, T>(item: &T) -> T
+pub fn strip_leading_trivia<T>(item: &T) -> T
 where
-    T: UpdateLeadingTrivia<'ast>,
+    T: UpdateLeadingTrivia,
 {
     item.update_leading_trivia(FormatTriviaType::Replace(vec![]))
 }
 
-pub fn strip_trailing_trivia<'ast, T>(item: &T) -> T
+pub fn strip_trailing_trivia<T>(item: &T) -> T
 where
-    T: UpdateTrailingTrivia<'ast>,
+    T: UpdateTrailingTrivia,
 {
     item.update_trailing_trivia(FormatTriviaType::Replace(vec![]))
 }
 
-pub trait UpdateLeadingTrivia<'ast> {
-    fn update_leading_trivia(&self, leading_trivia: FormatTriviaType<'ast>) -> Self;
+pub trait UpdateLeadingTrivia {
+    fn update_leading_trivia(&self, leading_trivia: FormatTriviaType) -> Self;
 }
 
-pub trait UpdateTrailingTrivia<'ast> {
-    fn update_trailing_trivia(&self, trailing_trivia: FormatTriviaType<'ast>) -> Self;
+pub trait UpdateTrailingTrivia {
+    fn update_trailing_trivia(&self, trailing_trivia: FormatTriviaType) -> Self;
 }
 
-pub trait UpdateTrivia<'ast> {
+pub trait UpdateTrivia {
     fn update_trivia(
         &self,
-        leading_trivia: FormatTriviaType<'ast>,
-        trailing_trivia: FormatTriviaType<'ast>,
+        leading_trivia: FormatTriviaType,
+        trailing_trivia: FormatTriviaType,
     ) -> Self;
 }
 
-impl<'ast, T> UpdateLeadingTrivia<'ast> for T
+impl<T> UpdateLeadingTrivia for T
 where
-    T: UpdateTrivia<'ast>,
+    T: UpdateTrivia,
 {
-    fn update_leading_trivia(&self, leading_trivia: FormatTriviaType<'ast>) -> Self
+    fn update_leading_trivia(&self, leading_trivia: FormatTriviaType) -> Self
     where
         Self: std::marker::Sized,
     {
@@ -70,11 +73,11 @@ where
     }
 }
 
-impl<'ast, T> UpdateTrailingTrivia<'ast> for T
+impl<T> UpdateTrailingTrivia for T
 where
-    T: UpdateTrivia<'ast>,
+    T: UpdateTrivia,
 {
-    fn update_trailing_trivia(&self, trailing_trivia: FormatTriviaType<'ast>) -> Self
+    fn update_trailing_trivia(&self, trailing_trivia: FormatTriviaType) -> Self
     where
         Self: std::marker::Sized,
     {
@@ -82,11 +85,11 @@ where
     }
 }
 
-impl<'ast> UpdateTrivia<'ast> for TokenReference<'ast> {
+impl UpdateTrivia for TokenReference {
     fn update_trivia(
         &self,
-        leading_trivia: FormatTriviaType<'ast>,
-        trailing_trivia: FormatTriviaType<'ast>,
+        leading_trivia: FormatTriviaType,
+        trailing_trivia: FormatTriviaType,
     ) -> Self {
         let added_leading_trivia = match leading_trivia {
             FormatTriviaType::Append(trivia) => {
@@ -117,11 +120,11 @@ impl<'ast> UpdateTrivia<'ast> for TokenReference<'ast> {
 
 macro_rules! define_update_trivia {
     ($node:ident, |$self:ident, $leading_trivia:ident, $trailing_trivia:ident| $body:expr) => {
-        define_update_trivia! {$node, |$self:&$node<'ast>, $leading_trivia: FormatTriviaType<'ast>, $trailing_trivia: FormatTriviaType<'ast>| $body}
+        define_update_trivia! {$node, |$self:&$node, $leading_trivia: FormatTriviaType, $trailing_trivia: FormatTriviaType| $body}
     };
     ($node:ident, $body:expr) => {
-        impl<'ast> UpdateTrivia<'ast> for $node<'ast> {
-            fn update_trivia(&self, leading_trivia: FormatTriviaType<'ast>, trailing_trivia: FormatTriviaType<'ast>) -> Self {
+        impl UpdateTrivia for $node {
+            fn update_trivia(&self, leading_trivia: FormatTriviaType, trailing_trivia: FormatTriviaType) -> Self {
                 $body(&self, leading_trivia, trailing_trivia)
             }
         }
@@ -130,11 +133,11 @@ macro_rules! define_update_trivia {
 
 macro_rules! define_update_leading_trivia {
     ($node:ident, |$self:ident, $leading_trivia:ident| $body:expr) => {
-        define_update_leading_trivia! {$node, |$self:&$node<'ast>, $leading_trivia: FormatTriviaType<'ast>| $body}
+        define_update_leading_trivia! {$node, |$self:&$node, $leading_trivia: FormatTriviaType| $body}
     };
     ($node:ident, $body:expr) => {
-        impl<'ast> UpdateLeadingTrivia<'ast> for $node<'ast> {
-            fn update_leading_trivia(&self, leading_trivia: FormatTriviaType<'ast>) -> Self {
+        impl UpdateLeadingTrivia for $node {
+            fn update_leading_trivia(&self, leading_trivia: FormatTriviaType) -> Self {
                 $body(&self, leading_trivia)
             }
         }
@@ -143,11 +146,11 @@ macro_rules! define_update_leading_trivia {
 
 macro_rules! define_update_trailing_trivia {
     ($node:ident, |$self:ident, $trailing_trivia:ident| $body:expr) => {
-        define_update_trailing_trivia! {$node, |$self:&$node<'ast>, $trailing_trivia: FormatTriviaType<'ast>| $body}
+        define_update_trailing_trivia! {$node, |$self:&$node, $trailing_trivia: FormatTriviaType| $body}
     };
     ($node:ident, $body:expr) => {
-        impl<'ast> UpdateTrailingTrivia<'ast> for $node<'ast> {
-            fn update_trailing_trivia(&self, trailing_trivia: FormatTriviaType<'ast>) -> Self {
+        impl UpdateTrailingTrivia for $node {
+            fn update_trailing_trivia(&self, trailing_trivia: FormatTriviaType) -> Self {
                 $body(&self, trailing_trivia)
             }
         }
@@ -312,7 +315,7 @@ define_update_trivia!(FunctionCall, |this, leading, trailing| {
         _ => this.prefix().update_leading_trivia(leading),
     };
 
-    let mut suffixes: Vec<Suffix<'ast>> = this.suffixes().map(|x| x.to_owned()).collect();
+    let mut suffixes: Vec<Suffix> = this.suffixes().map(|x| x.to_owned()).collect();
     match trailing {
         FormatTriviaType::NoChange => (),
         _ => {
@@ -384,11 +387,11 @@ define_update_leading_trivia!(Prefix, |this, leading| {
     }
 });
 
-impl<'ast, T> UpdateLeadingTrivia<'ast> for Punctuated<'ast, T>
+impl<T> UpdateLeadingTrivia for Punctuated<T>
 where
-    T: UpdateLeadingTrivia<'ast> + Clone,
+    T: UpdateLeadingTrivia + Clone,
 {
-    fn update_leading_trivia(&self, leading: FormatTriviaType<'ast>) -> Self {
+    fn update_leading_trivia(&self, leading: FormatTriviaType) -> Self {
         let mut punctuated = Punctuated::new();
         let mut pairs = self.to_owned().into_pairs();
 
@@ -410,11 +413,11 @@ where
     }
 }
 
-impl<'ast, T> UpdateTrailingTrivia<'ast> for Punctuated<'ast, T>
+impl<T> UpdateTrailingTrivia for Punctuated<T>
 where
-    T: UpdateTrailingTrivia<'ast> + Clone,
+    T: UpdateTrailingTrivia + Clone,
 {
-    fn update_trailing_trivia(&self, trailing: FormatTriviaType<'ast>) -> Self {
+    fn update_trailing_trivia(&self, trailing: FormatTriviaType) -> Self {
         let mut punctuated = self.to_owned();
 
         // Add any trailing trivia to the end of the punctuated list
@@ -474,6 +477,10 @@ define_update_leading_trivia!(Value, |this, leading| {
             Value::TableConstructor(table_constructor.update_leading_trivia(leading))
         }
         Value::Var(var) => Value::Var(var.update_leading_trivia(leading)),
+        #[cfg(feature = "luau")]
+        Value::IfExpression(if_expression) => {
+            Value::IfExpression(if_expression.update_leading_trivia(leading))
+        }
         other => panic!("unknown node {:?}", other),
     }
 });
@@ -503,6 +510,10 @@ define_update_trailing_trivia!(Value, |this, trailing| {
             Value::TableConstructor(table_constructor.update_trailing_trivia(trailing))
         }
         Value::Var(var) => Value::Var(var.update_trailing_trivia(trailing)),
+        #[cfg(feature = "luau")]
+        Value::IfExpression(if_expression) => {
+            Value::IfExpression(if_expression.update_trailing_trivia(trailing))
+        }
         other => panic!("unknown node {:?}", other),
     }
 });
@@ -533,7 +544,7 @@ define_update_trivia!(VarExpression, |this, leading, trailing| {
         _ => this.prefix().update_leading_trivia(leading),
     };
 
-    let mut suffixes: Vec<Suffix<'ast>> = this.suffixes().map(|x| x.to_owned()).collect();
+    let mut suffixes: Vec<Suffix> = this.suffixes().map(|x| x.to_owned()).collect();
     match trailing {
         FormatTriviaType::NoChange => (),
         _ => {
@@ -547,32 +558,47 @@ define_update_trivia!(VarExpression, |this, leading, trailing| {
 });
 
 #[cfg(feature = "luau")]
-define_update_trailing_trivia!(TypeInfo, |this, trailing| {
+define_update_trivia!(TypeInfo, |this, leading, trailing| {
     match this {
         TypeInfo::Array { braces, type_info } => TypeInfo::Array {
-            braces: braces.update_trailing_trivia(trailing),
+            braces: braces.update_trivia(leading, trailing),
             type_info: type_info.to_owned(),
         },
         TypeInfo::Basic(token_reference) => {
-            TypeInfo::Basic(token_reference.update_trailing_trivia(trailing))
+            TypeInfo::Basic(token_reference.update_trivia(leading, trailing))
         }
         TypeInfo::Callback {
+            generics,
             parentheses,
             arguments,
             arrow,
             return_type,
-        } => TypeInfo::Callback {
-            parentheses: parentheses.to_owned(),
-            arguments: arguments.to_owned(),
-            arrow: arrow.to_owned(),
-            return_type: Box::new(return_type.update_trailing_trivia(trailing)),
-        },
+        } => {
+            let (generics, parentheses) = if let Some(generics) = generics {
+                (
+                    Some(generics.update_leading_trivia(leading)),
+                    parentheses.to_owned(),
+                )
+            } else {
+                (
+                    generics.to_owned(),
+                    parentheses.update_leading_trivia(leading),
+                )
+            };
+            TypeInfo::Callback {
+                generics,
+                parentheses,
+                arguments: arguments.to_owned(),
+                arrow: arrow.to_owned(),
+                return_type: Box::new(return_type.update_trailing_trivia(trailing)),
+            }
+        }
         TypeInfo::Generic {
             base,
             arrows,
             generics,
         } => TypeInfo::Generic {
-            base: base.to_owned(),
+            base: base.update_leading_trivia(leading),
             arrows: arrows.update_trailing_trivia(trailing),
             generics: generics.to_owned(),
         },
@@ -582,7 +608,7 @@ define_update_trailing_trivia!(TypeInfo, |this, trailing| {
             ampersand,
             right,
         } => TypeInfo::Intersection {
-            left: left.to_owned(),
+            left: Box::new(left.update_leading_trivia(leading)),
             ampersand: ampersand.to_owned(),
             right: Box::new(right.update_trailing_trivia(trailing)),
         },
@@ -592,7 +618,7 @@ define_update_trailing_trivia!(TypeInfo, |this, trailing| {
             punctuation,
             type_info,
         } => TypeInfo::Module {
-            module: module.to_owned(),
+            module: module.update_leading_trivia(leading),
             punctuation: punctuation.to_owned(),
             type_info: Box::new(type_info.update_trailing_trivia(trailing)),
         },
@@ -601,12 +627,12 @@ define_update_trailing_trivia!(TypeInfo, |this, trailing| {
             base,
             question_mark,
         } => TypeInfo::Optional {
-            base: base.to_owned(),
+            base: Box::new(base.update_leading_trivia(leading)),
             question_mark: question_mark.update_trailing_trivia(trailing),
         },
 
         TypeInfo::Table { braces, fields } => TypeInfo::Table {
-            braces: braces.update_trailing_trivia(trailing),
+            braces: braces.update_trivia(leading, trailing),
             fields: fields.to_owned(),
         },
 
@@ -615,24 +641,24 @@ define_update_trailing_trivia!(TypeInfo, |this, trailing| {
             parentheses,
             inner,
         } => TypeInfo::Typeof {
-            typeof_token: typeof_token.to_owned(),
+            typeof_token: typeof_token.update_leading_trivia(leading),
             parentheses: parentheses.update_trailing_trivia(trailing),
             inner: inner.to_owned(),
         },
 
         TypeInfo::Tuple { parentheses, types } => TypeInfo::Tuple {
-            parentheses: parentheses.update_trailing_trivia(trailing),
+            parentheses: parentheses.update_trivia(leading, trailing),
             types: types.to_owned(),
         },
 
         TypeInfo::Union { left, pipe, right } => TypeInfo::Union {
-            left: left.to_owned(),
+            left: Box::new(left.update_leading_trivia(leading)),
             pipe: pipe.to_owned(),
             right: Box::new(right.update_trailing_trivia(trailing)),
         },
 
         TypeInfo::Variadic { ellipse, type_info } => TypeInfo::Variadic {
-            ellipse: ellipse.to_owned(),
+            ellipse: ellipse.update_leading_trivia(leading),
             type_info: Box::new(type_info.update_trailing_trivia(trailing)),
         },
 
@@ -660,20 +686,85 @@ define_update_trailing_trivia!(IndexedTypeInfo, |this, trailing| {
     }
 });
 
+#[cfg(feature = "luau")]
+define_update_leading_trivia!(TypeArgument, |this, leading| {
+    if let Some((name, colon)) = this.name() {
+        this.to_owned().with_name(Some((
+            name.update_leading_trivia(leading),
+            colon.to_owned(),
+        )))
+    } else {
+        this.to_owned()
+            .with_type_info(this.type_info().update_leading_trivia(leading))
+    }
+});
+
 #[cfg(feature = "luau")]
 define_update_trailing_trivia!(TypeAssertion, |this, trailing| {
     this.to_owned()
         .with_cast_to(this.cast_to().update_trailing_trivia(trailing))
 });
 
+#[cfg(feature = "luau")]
+define_update_leading_trivia!(TypeField, |this, leading| {
+    this.to_owned()
+        .with_key(this.key().update_leading_trivia(leading))
+});
+
 #[cfg(feature = "luau")]
 define_update_trailing_trivia!(TypeField, |this, trailing| {
     this.to_owned()
         .with_value(this.value().update_trailing_trivia(trailing))
 });
 
+#[cfg(feature = "luau")]
+define_update_leading_trivia!(TypeFieldKey, |this, leading| {
+    match this {
+        TypeFieldKey::Name(token) => TypeFieldKey::Name(token.update_leading_trivia(leading)),
+        TypeFieldKey::IndexSignature { brackets, inner } => TypeFieldKey::IndexSignature {
+            brackets: brackets.update_leading_trivia(leading),
+            inner: inner.to_owned(),
+        },
+        other => panic!("unknown node {:?}", other),
+    }
+});
+
 #[cfg(feature = "luau")]
 define_update_trailing_trivia!(TypeSpecifier, |this, trailing| {
     this.to_owned()
         .with_type_info(this.type_info().update_trailing_trivia(trailing))
 });
+
+#[cfg(feature = "luau")]
+define_update_leading_trivia!(GenericDeclaration, |this, leading| {
+    this.to_owned()
+        .with_arrows(this.arrows().update_leading_trivia(leading))
+});
+
+#[cfg(feature = "luau")]
+define_update_leading_trivia!(GenericDeclarationParameter, |this, leading| {
+    match this {
+        GenericDeclarationParameter::Name(token) => {
+            GenericDeclarationParameter::Name(token.update_leading_trivia(leading))
+        }
+        GenericDeclarationParameter::Variadic { name, ellipse } => {
+            GenericDeclarationParameter::Variadic {
+                name: name.update_leading_trivia(leading),
+                ellipse: ellipse.to_owned(),
+            }
+        }
+        other => panic!("unknown node {:?}", other),
+    }
+});
+
+#[cfg(feature = "luau")]
+define_update_leading_trivia!(IfExpression, |this, leading| {
+    this.to_owned()
+        .with_if_token(this.if_token().update_leading_trivia(leading))
+});
+
+#[cfg(feature = "luau")]
+define_update_trailing_trivia!(IfExpression, |this, trailing| {
+    this.to_owned()
+        .with_else(this.else_expression().update_trailing_trivia(trailing))
+});
